import type { ExceptionMessage } from "#app/exceptions/handler.ts"
import Page, { type PageWithInternalLinksRaw } from "#app/models/page.ts"
import { throttle } from "#start/limiter.ts"
import type { HttpContext } from "@adonisjs/core/http"
import router from "@adonisjs/core/services/router"
import vine from "@vinejs/vine"

export const get_wikipedia_page_by_id_validator = vine.compile(
  vine.object({
    params: vine.object({
      id: vine.number().withoutDecimals().positive(),
    }),
  }),
)

export default class get_wikipedia_page_by_id {
  public async handle(context: HttpContext): Promise<
    | {
        __response: ExceptionMessage
        __status: 404
      }
    | {
        __response: PageWithInternalLinksRaw
        __status: 200
      }
  > {
    const payload = await context.request.validateUsing(
      get_wikipedia_page_by_id_validator,
    )
    const page = await Page.findOrFail(payload.params.id)
    await page.load("internalLinks")
    return context.response.ok(page)
  }
}

router
  .get("/wikipedia/pages/:id", [get_wikipedia_page_by_id])
  .use(throttle)
  .openapi({
    description: "Get a Wikipedia page by ID.",
    tags: ["wikipedia"],
  })
